@@ -21,14 +21,14 @@ stripMatrices <- function(matrices, fill = -1) {
 
 
 removeInteractions <- function(model) {
-  model$matrices$OmegaEtaXi[TRUE] <- 0 
+  model$matrices$OmegaEtaXi[TRUE] <- 0
   model$matrices$OmegaXiXi[TRUE] <- 0
   model
 }
 
 
-# Faster version of mvtnorm::dmvnorm() given that sigma is positive 
-# there are some drawbacks to using mvnfast. In particular, 
+# Faster version of mvtnorm::dmvnorm() given that sigma is positive
+# there are some drawbacks to using mvnfast. In particular,
 # its a little less consistent
 dmvn <- function(X, mean, sigma, log = FALSE) {
   return(tryCatch(mvnfast::dmvn(X, mean, sigma, log, ncores = 2), #ThreadEnv$n.threads),
@@ -38,19 +38,19 @@ dmvn <- function(X, mean, sigma, log = FALSE) {
 
 diagPartitionedMat <- function(X, Y) {
   if (is.null(X)) return(Y) else if (is.null(Y)) return(X)
-  structure(rbind(cbind(X, matrix(0, nrow = NROW(X), ncol = NCOL(Y))), 
+  structure(rbind(cbind(X, matrix(0, nrow = NROW(X), ncol = NCOL(Y))),
                cbind(matrix(0, nrow = NROW(Y), ncol = NCOL(X)), Y)),
-            dimnames = list(c(rownames(X), rownames(Y)), 
+            dimnames = list(c(rownames(X), rownames(Y)),
                             c(colnames(X), colnames(Y))))
 }
 
 
 formatNumeric <- function(x, digits = 3) {
   if (is.numeric(x)) {
-    format(round(x, digits), nsmall = digits, digits = digits, 
+    format(round(x, digits), nsmall = digits, digits = digits,
            trim = FALSE, justify = "right")
   } else {
-    format(x, trim = FALSE, justify = "right") 
+    format(x, trim = FALSE, justify = "right")
   }
 }
 
@@ -76,8 +76,13 @@ getK_NA <- function(omega, labelOmega) {
 
 
 sortData <- function(data, allIndsXis, allIndsEtas) {
-  if (!all(c(allIndsXis, allIndsEtas) %in% colnames(data))) 
-    stop2("Missing Observed Variables in Data")
+  inds    <- c(allIndsXis, allIndsEtas)
+  ovs     <- colnames(data)
+  missing <- inds[!inds %in% ovs]
+
+  stopif(!all(inds %in% ovs), "Missing observed variables in data:\n  ",
+         missing)
+
   data[c(allIndsXis, allIndsEtas)]
 }
 
@@ -88,15 +93,16 @@ anyAllNA <- function(data) {
 
 
 castDataNumericMatrix <- function(data) {
+  force(data) # evaluate to check for errors
   data <- tryCatch({
     numericData <- lapplyDf(data, FUN = as.numeric)
   },
   warning = function(w) {
     warning2("Warning in converting data to numeric: \n", w)
     numericData <- suppressWarnings(lapplyDf(data, FUN = as.numeric))
-    if (anyAllNA(numericData)) stop2("Unable to conver data to type numeric") 
+    stopif(anyAllNA(numericData), "Unable to convert data to type numeric")
     numeric
-  }, 
+  },
   error = function(e) {
     stop2("Unable to convert data to type numeric")
   })
@@ -114,7 +120,7 @@ filterData <- function(data) {
 cleanAndSortData <- function(data, allIndsXis, allIndsEtas) {
   if (is.null(data)) return(NULL)
   # sort Data before optimizing starting params
-  sortData(data, allIndsXis,  allIndsEtas) |> 
+  sortData(data, allIndsXis,  allIndsEtas) |>
     castDataNumericMatrix() |> filterData()
 }
 
@@ -135,7 +141,7 @@ createDoubleIntTerms <- function(x, z = NULL, sep = ":") {
 
 
 getFreeOrConstIntTerms <- function(varsInInt, eta, intTerms) {
-  expr <- intTerms[intTerms$lhs == eta & intTerms$rhs %in% 
+  expr <- intTerms[intTerms$lhs == eta & intTerms$rhs %in%
                    createDoubleIntTerms(varsInInt), "mod"]
   if (canBeNumeric(expr, includeNA = TRUE)) return(as.numeric(expr))
   0
@@ -143,14 +149,14 @@ getFreeOrConstIntTerms <- function(varsInInt, eta, intTerms) {
 
 
 getLabelIntTerms <- function(varsInInt, eta, intTerms) {
-  expr <- intTerms[intTerms$lhs == eta & intTerms$rhs %in% 
+  expr <- intTerms[intTerms$lhs == eta & intTerms$rhs %in%
                    createDoubleIntTerms(varsInInt), "mod"]
   if (!canBeNumeric(expr)) return(expr)
   ""
 }
 
 
-getEmptyModel <- function(parTable, cov.syntax, parTableCovModel, 
+getEmptyModel <- function(parTable, cov.syntax, parTableCovModel,
                           method = "lms") {
   parTable$mod <- ""
   parTable <- removeConstraintExpressions(parTable)
@@ -201,7 +207,7 @@ getLabeledParamsLavaan <- function(parTable, fixedParams = NULL) {
                         !parTable$label %in% fixedParams,
                         c("est", "label"), drop = FALSE] |>
     uniqueByVar("label")
-  
+
   theta <- as.numeric(labelRows$est)
   names(theta) <- labelRows$label
   theta
@@ -221,68 +227,8 @@ uniqueByVar <- function(df, var) {
 }
 
 
-checkModel <- function(model, covModel = NULL, method = "lms") {
-  modelXis <- model$info$xis
-  if (!is.null(covModel$info)) {
-    covModelEtas <- covModel$info$etas
-    covModelXis <- covModel$info$xis 
-    if (!all(c(covModelXis, covModelEtas) %in% modelXis)) {
-      stop2("All latent variables in the cov-model must be an exogenous variable in the main model")
-    }
-    if (!all(modelXis %in% c(covModelXis, covModelEtas))) {
-      stop2("All exogenous variables in main model must be part of the cov-model")
-    }
-  }
-
-  checkNodesLms(parTable = rbind(model$parTable, covModel$parTable),
-                nodes = model$quad$m, method = method)
-  NULL
-}
-
-
-checkNodesLms <- function(parTable,
-                          nodes, 
-                          method = "lms",
-                          minNodesXiXi = 16,
-                          minNodesXiEta = 32, 
-                          minNodesEtaEta = 48) {
-  if (method == "qml") return(NULL)
-
-  etas <- getEtas(parTable, isLV = TRUE)
-  xis <- getXis(parTable, etas = etas, isLV = TRUE)
-  varsInts <- getVarsInts(getIntTermRows(parTable))
-
-  nodesXiXi_ok <- TRUE
-  nodesXiEta_ok <- TRUE
-  nodesEtaEta_ok <- TRUE
-  
-  lapply(varsInts, FUN = function(x) {
-    if (all(x %in% xis)) nodesXiXi_ok <<- nodes >= minNodesXiXi
-    else if (all(x %in% etas)) nodesEtaEta_ok <<- nodes >= minNodesEtaEta
-    else if (any(x %in% etas)) nodesXiEta_ok <<- nodes >= minNodesXiEta
-    else warning2("Unable to classify latent variables in interaction terms")
-  }) 
-
-  if (!nodesXiXi_ok) {
-    warning2("It is recommended that you have at least ", minNodesXiXi,  " nodes ",
-             "for interaction effects between exogenous variables in the lms approach ", 
-             "'nodes = ", nodes, "'")
-  } 
-  if (!nodesXiEta_ok) {
-    warning2("It is recommended that you have at least ", minNodesXiEta, " nodes ", 
-             "for interaction effects between exogenous and endogenous variables in the lms approach ", 
-             "'nodes = ", nodes, "'")
-  }   
-  if (!nodesEtaEta_ok) {
-    warning2("It is recommended that you have at least ", minNodesEtaEta, " nodes ",
-             "for interaction effects between endogenous variables in the lms approach ", 
-             "'nodes = ", nodes, "'")
-  }
-}
-
-
 removeInteractionVariances <- function(parTable) {
-  parTable[!(parTable$op == "~~" & 
+  parTable[!(parTable$op == "~~" &
              (grepl(":", parTable$lhs) | grepl(":", parTable$rhs))), ]
 }
 
@@ -291,26 +237,26 @@ tr <- function(mat) sum(diag(mat))
 
 
 traceOmegaXiXi <- function(omega, numEta, numXi) {
-  lastRow <- 0 
-  lastCol <- 0  
+  lastRow <- 0
+  lastCol <- 0
   trace <- numeric(numEta)
   for (i in seq_len(numEta)) {
-    trace[[i]] <- tr(omega[seq_len(numXi) + (i - 1) * numXi, 
-                           seq_len(numXi) + (i - 1) * numXi]) 
+    trace[[i]] <- tr(omega[seq_len(numXi) + (i - 1) * numXi,
+                           seq_len(numXi) + (i - 1) * numXi])
   }
   trace
 }
 
 
 diagPartitioned <- function(matrix, length) {
-  out <- matrix(0, nrow = length * nrow(matrix), 
+  out <- matrix(0, nrow = length * nrow(matrix),
                 ncol = length * ncol(matrix))
   nrows <- nrow(matrix)
   rows <- seq_len(nrows)
   ncols <- ncol(matrix)
   cols <- seq_len(ncols)
   for (i in seq_len(length)) {
-    out[rows + (i - 1) * nrows, 
+    out[rows + (i - 1) * nrows,
         cols + (i - 1) * ncols] <- matrix
   }
   out
@@ -319,7 +265,7 @@ diagPartitioned <- function(matrix, length) {
 
 repPartitionedRows <- function(matrix, length = 1) {
   if (length <= 1) return(matrix)
-  out <- matrix 
+  out <- matrix
   for (i in seq_len(length - 1)) {
     out <- rbind(out, matrix)
   }
@@ -329,7 +275,7 @@ repPartitionedRows <- function(matrix, length = 1) {
 
 repPartitionedCols <- function(matrix, length = 1) {
   if (length <= 1) return(matrix)
-  out <- matrix 
+  out <- matrix
   for (i in seq_len(length - 1)) {
     out <- cbind(out, matrix)
   }
@@ -338,7 +284,7 @@ repPartitionedCols <- function(matrix, length = 1) {
 
 
 diagBindSquareMatrices <- function(X, Y) {
-  XY <- matrix(0, nrow = NROW(X), ncol = NCOL(Y), 
+  XY <- matrix(0, nrow = NROW(X), ncol = NCOL(Y),
                dimnames = list(rownames(X), colnames(Y)))
   rbind(cbind(X, XY), cbind(t(XY), Y))
 }
@@ -346,8 +292,8 @@ diagBindSquareMatrices <- function(X, Y) {
 
 #' @export
 as.logical.matrix <- function(x, ...) {
-  structure(x != 0, 
-            dim = dim(x), 
+  structure(x != 0,
+            dim = dim(x),
             dimnames = dimnames(x))
 }
 
@@ -359,7 +305,7 @@ isScalingY <- function(x) {
 
 runningAverage <- function(x, n = 10) {
   if (length(x) < n) return(0)
-  last <- length(x)  
+  last <- length(x)
   if (last < n) first <- 1 else first <- last - n + 1
   mean(x[first:last])
 }
@@ -367,7 +313,7 @@ runningAverage <- function(x, n = 10) {
 
 nNegativeLast <- function(x, n = 10) {
   if (length(x) < n) return(0)
-  last <- length(x)  
+  last <- length(x)
   if (last < n) first <- 1 else first <- last - n + 1
   sum(x[first:last] < 0)
 }
@@ -382,7 +328,7 @@ getDegreesOfFreedom <- function(m, coef) {
 
 
 getInfoQuad <- function(quad) {
-  list(dim = quad$k, nodes.dim = quad$m, nodes.total = quad$m ^ quad$k) 
+  list(dim = quad$k, nodes.dim = quad$m, nodes.total = quad$m ^ quad$k)
 }
 
 
