@@ -5,7 +5,7 @@ m1 <- '
 X =~ x1 + x2 + x3
 Y =~ y1 + y2 + y3
 Z =~ z1 + z2 + z3
-Y ~ X + Z + X:Z
+Y ~ X + Z + a * X:Z
 '
 
 est1 <- modsem(m1, data = oneInt, convergence = 1e-2, method = "qml",
@@ -21,10 +21,14 @@ tpb <- '
   INT =~ int1 + int2 + int3
   BEH =~ b1 + b2
 
+  # Covariances
+  ATT ~~ SN + PBC
+  PBC ~~ SN 
 # Inner Model (Based on Steinmetz et al., 2011)
-  BEH ~ INT + PBC 
-  INT ~ ATT + SN + PBC
+  BEH ~ b * INT + PBC 
+  INT ~ ATT + SN + a * PBC
   BEH ~ PBC:INT
+  VERY_LONG_LABEL_THAT_SHOULD_BE_SHORTENED := a * b
 '
 
 est2 <- modsem(tpb, data = TPB, method = "qml", 
